Package: OHPL
Type: Package
Title: Ordered Homogeneity Pursuit Lasso for Group Variable Selection
Version: 1.4.2
Authors@R: c(
    person("You-Wu", "Lin", email = "lyw015813@126.com", role = c("aut")),
    person("Nan", "Xiao", email = "me@nanx.me", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0250-5673")))
Description: Ordered homogeneity pursuit lasso (OHPL)
    algorithm for group variable selection proposed in Lin et al. (2017)
    <DOI:10.1016/j.chemolab.2017.07.004>. The OHPL method exploits the
    homogeneity structure in high-dimensional data and enjoys the
    grouping effect to select groups of important variables
    automatically. This feature makes it particularly useful for
    high-dimensional datasets with strongly correlated variables,
    such as spectroscopic data.
License: GPL-3 | file LICENSE
URL: https://ohpl.io, https://ohpl.io/doc/,
        https://github.com/nanxstats/OHPL
BugReports: https://github.com/nanxstats/OHPL/issues
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.0.2)
Imports: glmnet, mvtnorm, pls
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-13 00:17:57 UTC; nanx
Author: You-Wu Lin [aut],
  Nan Xiao [aut, cre] (ORCID: <https://orcid.org/0000-0002-0250-5673>)
Maintainer: Nan Xiao <me@nanx.me>
Repository: CRAN
Date/Publication: 2026-04-13 11:40:13 UTC
Built: R 4.5.2; ; 2026-04-15 05:00:07 UTC; unix

Package: OHPL
Type: Package
Title: Ordered Homogeneity Pursuit Lasso for Group Variable Selection
Version: 1.4.2
Authors@R: c(
    person("You-Wu", "Lin", email = "lyw015813@126.com", role = c("aut")),
    person("Nan", "Xiao", email = "me@nanx.me", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0250-5673")))
Description: Ordered homogeneity pursuit lasso (OHPL)
    algorithm for group variable selection proposed in Lin et al. (2017)
    <DOI:10.1016/j.chemolab.2017.07.004>. The OHPL method exploits the
    homogeneity structure in high-dimensional data and enjoys the
    grouping effect to select groups of important variables
    automatically. This feature makes it particularly useful for
    high-dimensional datasets with strongly correlated variables,
    such as spectroscopic data.
License: GPL-3 | file LICENSE
URL: https://ohpl.io, https://ohpl.io/doc/,
        https://github.com/nanxstats/OHPL
BugReports: https://github.com/nanxstats/OHPL/issues
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.0.2)
Imports: glmnet, mvtnorm, pls
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-13 00:17:57 UTC; nanx
Author: You-Wu Lin [aut],
  Nan Xiao [aut, cre] (ORCID: <https://orcid.org/0000-0002-0250-5673>)
Maintainer: Nan Xiao <me@nanx.me>
Repository: CRAN
Date/Publication: 2026-04-13 11:40:13 UTC
Built: R 4.6.0; ; 2026-04-26 09:11:39 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(predict,OHPL)
export(FOP)
export(OHPL)
export(OHPL.RMSEP)
export(OHPL.sim)
export(cv.OHPL)
export(dlc)
export(proto)
import(pls)
importFrom(glmnet,cv.glmnet)
importFrom(mvtnorm,rmvnorm)
importFrom(stats,coef)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,rnorm)
importFrom(stats,sd)

# Ordered Homogeneity Pursuit Lasso <img src="man/figures/logo.png" align="right" width="120" />

<!-- badges: start -->
[![R-CMD-check](https://github.com/nanxstats/OHPL/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/nanxstats/OHPL/actions/workflows/R-CMD-check.yaml)
[![CRAN Version](https://www.r-pkg.org/badges/version/OHPL)](https://cran.r-project.org/package=OHPL)
[![Downloads from the RStudio CRAN mirror](https://cranlogs.r-pkg.org/badges/OHPL)](https://cran.r-project.org/package=OHPL)
<!-- badges: end -->

## Introduction

Implements the ordered homogeneity pursuit lasso (OHPL) algorithm for group variable selection proposed in Lin et al. (2017) <[DOI:10.1016/j.chemolab.2017.07.004](https://doi.org/10.1016/j.chemolab.2017.07.004)> ([PDF](https://nanx.me/papers/OHPL.pdf)). The OHPL method exploits the homogeneity structure in high-dimensional data and enjoys the grouping effect to select groups of important variables automatically. This feature makes it particularly useful for high-dimensional datasets with strongly correlated variables, such as spectroscopic data.

## Paper citation

Formatted citation:

You-Wu Lin, Nan Xiao, Li-Li Wang, Chuan-Quan Li, and Qing-Song Xu (2017). Ordered homogeneity pursuit lasso for group variable selection with applications to spectroscopic data. _Chemometrics and Intelligent Laboratory Systems_ 168, 62-71.

BibTeX entry:

```
@article{lin2017ordered,
  title   = {Ordered homogeneity pursuit lasso for group variable selection with applications to spectroscopic data},
  author  = {You-Wu Lin and Nan Xiao and Li-Li Wang and Chuan-Quan Li and Qing-Song Xu},
  journal = {Chemometrics and Intelligent Laboratory Systems},
  year    = {2017},
  volume  = {168},
  pages   = {62--71},
  doi     = {10.1016/j.chemolab.2017.07.004}
}
```

## Installation

You can install OHPL from CRAN:

```r
install.packages("OHPL")
```

Or try the development version on GitHub:

```r
# install.packages("remotes")
remotes::install_github("nanxstats/OHPL")
```

To get started, try the examples in `OHPL()`:

```r
library("OHPL")
?OHPL
```

[Browse the package documentation](https://ohpl.io/doc/) for more information.

## Contribute

To contribute to this project, please take a look at the [Contributing Guidelines](https://ohpl.io/doc/CONTRIBUTING.html) first.
Please note that the OHPL project is released with a
[Contributor Code of Conduct](https://ohpl.io/doc/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

#' Make predictions based on the fitted OHPL model
#'
#' Make predictions on new data by an OHPL model object.
#'
#' @param object Object of class `OHPL` fitted by [OHPL()].
#' @param newx Predictor matrix of the new data.
#' @param ncomp Optimal number of components.
#'   If is `NULL`, the optimal number of components
#'   stored in the model object will be used.
#' @param type Prediction type.
#' @param ... Additional parameters.
#'
#' @return Numeric matrix of the predicted values.
#'
#' @method predict OHPL
#'
#' @importFrom stats predict
#'
#' @export
#'
#' @examples
#' # generate simulation data
#' dat <- OHPL.sim(
#'   n = 100, p = 100, rho = 0.8,
#'   coef = rep(1, 10), snr = 3, p.train = 0.5,
#'   seed = 1010
#' )
#'
#' # split training and test set
#' x <- dat$x.tr
#' y <- dat$y.tr
#' x.test <- dat$x.te
#' y.test <- dat$y.te
#'
#' # fit the OHPL model
#' fit <- OHPL(x, y, maxcomp = 3, gamma = 0.5, G = 10, type = "max")
#' # make predictions
#' y.pred <- predict(fit, x.test)
#' y.pred
predict.OHPL <- function(object, newx, ncomp = NULL, type = "response", ...) {
  if (!("OHPL" %in% class(object))) stop('object class must be "OHPL"')

  ncomp <- ifelse(is.null(ncomp), object$"opt.K", ncomp)

  y.pred <- predict(
    object$"model",
    newdata = newx[, object$"Vsel"],
    ncomp = ncomp, type = type
  )
  y.pred <- as.matrix(y.pred, ncol = 1L)

  y.pred
}

#' The beer dataset
#'
#' The beer dataset contains 60 samples published by Norgaard et al.
#' Recorded with a 30mm quartz cell on the undiluted degassed beer
#' and measured from 1100 to 2250 nm (576 data points) in steps of 2 nm.
#'
#' @docType data
#' @name beer
#' @usage data(beer)
#'
#' @references
#' Norgaard, L., Saudland, A., Wagner, J., Nielsen, J. P., Munck, L., &
#' Engelsen, S. B. (2000). Interval partial least-squares regression (iPLS):
#' a comparative chemometric study with an example from near-infrared
#' spectroscopy. _Applied Spectroscopy_, 54(3), 413--419.
#'
#' @examples
#' data("beer")
#' x.cal <- beer$xtrain
#' dim(x.cal)
#' x.test <- beer$xtest
#' dim(x.test)
#' y.cal <- beer$ytrain
#' dim(y.cal)
#' y.test <- beer$ytest
#' dim(y.test)
#'
#' X <- rbind(x.cal, x.test)
#' y <- rbind(y.cal, y.test)
#' n <- nrow(y)
#'
#' set.seed(1001)
#' samp.idx <- sample(1L:n, round(n * 0.7))
#' X.cal <- X[samp.idx, ]
#' y.cal <- y[samp.idx]
#' X.test <- X[-samp.idx, ]
#' y.test <- y[-samp.idx]
NULL

#' The wheat dataset
#'
#' The wheat dataset contains 100 wheat samples with specified
#' protein and moisture content, published by J. Kalivas.
#' Samples were measured by diffuse reflectance as log (I/R)
#' from 1100 to 2500 nm (701 data points) in 2 nm intervals.
#'
#' @docType data
#' @name wheat
#' @usage data(wheat)
#'
#' @references
#' Kalivas, J. H. (1997). Two data sets of near infrared spectra.
#' _Chemometrics and Intelligent Laboratory Systems_, 37(2), 255--259.
#'
#' @examples
#' data("wheat")
#'
#' X <- wheat$x
#' y <- wheat$protein
#' n <- nrow(wheat$x)
#'
#' set.seed(1001)
#' samp.idx <- sample(1L:n, round(n * 0.7))
#' X.cal <- X[samp.idx, ]
#' y.cal <- y[samp.idx]
#' X.test <- X[-samp.idx, ]
#' y.test <- y[-samp.idx]
NULL

#' The soil dataset
#'
#' The soil dataset contains 108 sample measurements from the
#' wavelength range of 400–2500 nm (visible and near infrared spectrum)
#' published by Rinnan et al.
#'
#' @docType data
#' @name soil
#' @usage data(soil)
#'
#' @references
#' Rinnan, R., & Rinnan, A. (2007). Application of near infrared
#' reflectance (NIR) and fluorescence spectroscopy to analysis of
#' microbiological and chemical properties of arctic soil.
#' _Soil biology and Biochemistry_, 39(7), 1664--1673.
#'
#' @examples
#' data("soil")
#'
#' X <- soil$x
#' y <- soil$som
#' n <- nrow(soil$x)
#'
#' set.seed(1001)
#' samp.idx <- sample(1L:n, round(n * 0.7))
#' X.cal <- X[samp.idx, ]
#' y.cal <- y[samp.idx]
#' X.test <- X[-samp.idx, ]
#' y.test <- y[-samp.idx]
NULL

#' Generate simulation data for benchmarking sparse regressions
#' (Gaussian response)
#'
#' Generate simulation data (Gaussian case) following the
#' settings in Xiao and Xu (2015).
#'
#' @param n Number of observations.
#' @param p Number of variables.
#' @param rho Correlation base for generating correlated variables.
#' @param coef Vector of non-zero coefficients.
#' @param snr Signal-to-noise ratio (SNR).
#' @param p.train Percentage of training set.
#' @param seed Random seed for reproducibility.
#'
#' @return A list containing `x.tr`, `x.te`, `y.tr`, and `y.te`.
#'
#' @references
#' Nan Xiao and Qing-Song Xu. (2015). Multi-step adaptive elastic-net:
#' reducing false positives in high-dimensional variable selection.
#' _Journal of Statistical Computation and Simulation_ 85(18), 3755--3765.
#'
#' @importFrom mvtnorm rmvnorm
#' @importFrom stats rnorm
#'
#' @export OHPL.sim
#'
#' @examples
#' dat <- OHPL.sim(
#'   n = 100, p = 100, rho = 0.8,
#'   coef = rep(1, 10), snr = 3, p.train = 0.5,
#'   seed = 1010
#' )
#'
#' dim(dat$x.tr)
#' dim(dat$x.te)
OHPL.sim <- function(
    n = 100, p = 100,
    rho = 0.8, coef = rep(1, 10), snr = 3,
    p.train = 0.5, seed = 1001) {
  call <- match.call()

  set.seed(seed)

  sigma <- matrix(0, p, p)
  corvec <- function(i, p, rho) rho^(abs(i - 1L:p))
  for (i in 1:p) sigma[i, ] <- corvec(i, p, rho)

  X <- rmvnorm(n, rep(0, p), sigma)

  # non-zero coefficients
  beta0 <- matrix(c(coef, rep(0, (p - length(coef)))))

  snr.numerator <- as.vector(t(beta0) %*% sigma %*% beta0)
  snr.denominator <- snr.numerator / snr
  sd <- sqrt(snr.denominator)
  eps <- matrix(rnorm(n, 0, sd))

  y <- as.matrix((X %*% beta0) + eps)

  # training / test set splitting
  tr.row <- sample(1L:n, round(n * p.train), replace = FALSE)

  x.tr <- X[tr.row, , drop = FALSE]
  y.tr <- y[tr.row, , drop = FALSE]
  x.te <- X[-tr.row, , drop = FALSE]
  y.te <- y[-tr.row, , drop = FALSE]

  list(
    "x.tr" = x.tr, "y.tr" = y.tr,
    "x.te" = x.te, "y.te" = y.te,
    "call" = call
  )
}

single.beta <- function(X, y, maxcomp = NULL) {
  if (missing(X) | missing(y)) stop("Please specify both x and y")
  if (is.null(maxcomp)) maxcomp <- min(nrow(X) - 1L, ncol(X))

  plsdf <- as.data.frame(cbind(X, "y" = y))

  plsr.cvfit <- pls::plsr(
    y ~ X,
    data = plsdf, ncomp = maxcomp,
    scale = TRUE, method = "simpls", validation = "LOO"
  )

  # Select the best component number using adjusted CV
  opt.cv <- which.min(pls::RMSEP(plsr.cvfit)[["val"]][2L, 1L, -1L])
  rmse.cv <- min(pls::RMSEP(plsr.cvfit)[["val"]][2L, 1L, -1L])

  plsr.fit <- pls::plsr(
    y ~ X,
    data = plsdf, ncomp = opt.cv,
    scale = TRUE, method = "simpls", validation = "none"
  )

  beta1 <- matrix(coef(plsr.fit), ncol = 1L)

  simplsfit <- simpls.fit(X, y, opt.cv)
  beta <- coef(simplsfit, ncomp = opt.cv, intercept = FALSE)
  beta.simpls <- matrix(simplsfit$coefficients[, , opt.cv], ncol = 1L)

  obj <- list(
    "beta" = beta1,
    "opt.cv" = opt.cv,
    "rmse.cv" = rmse.cv,
    "beta.simpls" = beta.simpls
  )

  obj
}

#' Compute D, L, and C in the Fisher optimal partitions algorithm
#'
#' @param X A set of samples.
#' @param maxk Maximum number of `k`.
#'
#' @return The D, L, and C statistics in the
#'   Fisher optimal partitions algorithm.
#'
#' @export dlc
dlc <- function(X, maxk) {
  n <- dim(X)[1]
  D <- matrix(0, n, n)
  rownames(D) <- 1:n
  colnames(D) <- 1:n

  result <- sapply(1:(n - 1), function(x) dia(X, x, n))
  result <- do.call("c", result)
  D[lower.tri(D)] <- result
  D <- t(D)
  D[lower.tri(D)] <- result

  L <- matrix(0, n - 1, maxk - 1)
  rownames(L) <- 2:n
  colnames(L) <- 2:maxk

  C <- matrix(0, n - 1, maxk - 1)
  rownames(C) <- 2:n
  colnames(C) <- 2:maxk
  diag(C) <- 2:maxk

  lc <- sapply(
    3:n,
    function(x) {
      c(
        min(D[1, 1:(x - 1)] + D[2:x, x]),
        which.min(D[1, 1:(x - 1)] + D[2:x, x])[1] + 1
      )
    }
  )

  L[as.character(3:n), "2"] <- lc[1, ]
  C[as.character(3:n), "2"] <- lc[2, ]

  for (j in 3:maxk) {
    lc <- sapply(
      (j + 1):n,
      function(x) {
        c(
          min(L[(j:x) - 2, j - 2] + D[j:x, x]),
          which.min(L[(j:x) - 2, j - 2] + D[j:x, x])[1] + j - 1
        )
      }
    )
    L[as.character((j + 1):n), as.character(j)] <- lc[1, ]
    C[as.character((j + 1):n), as.character(j)] <- lc[2, ]
    # cat("class", j, "of L, C of", type, "finished in dlc", "\n")
  }

  list("D" = D, "L" = L, "C" = C)
}

#' Fisher optimal partition
#'
#' The Fisher optimal partition algorithm.
#'
#' @param X A set of samples.
#' @param k Number of classes.
#' @param C Statistic from the output of [dlc()].
#'
#' @return Index vector for each sample's classification.
#'
#' @export FOP
#'
#' @references
#' W. D. Fisher (1958). On grouping for maximum homogeneity.
#' _Journal of the American Statistical Association_,
#' vol. 53, pp. 789--798.
#'
#' @examples
#' X <- matrix(c(
#'   9.3, 1.8, 1.9, 1.7, 1.5, 1.3,
#'   1.4, 2.0, 1.9, 2.3, 2.1
#' ))
#' C <- dlc(X, maxk = 8)$C
#' F <- FOP(X, 8, C)
FOP <- function(X, k, C) {
  n <- dim(X)[1]
  if (k == 1) {
    idx <- rep(1, n)
  } else {
    breaks <- rep(0, k)
    breaks[1] <- 1
    breaks[k] <- C[n - 1, k - 1]
    idx <- rep(1, n)
    idx[breaks[k]:n] <- k
    if (k > 2) {
      for (flag in (k - 1):2) {
        breaks[flag] <- C[breaks[flag + 1] - 2, flag - 1]
        idx[breaks[flag]:(breaks[flag + 1] - 1)] <- flag
      }
    }
  }

  idx
}

#' Compute RMSEP, MAE, and Q2 for a test set
#'
#' Makes predictions on new data and computes the performance evaluation
#' metrics RMSEP, MAE, and Q2.
#'
#' @param object Object of class `OHPL` fitted by [OHPL()].
#' @param newx Predictor matrix of the new data.
#' @param newy Response matrix of the new data (matrix with one column).
#'
#' @return A list of the performance metrics.
#'
#' @export OHPL.RMSEP
#'
#' @examples
#' # Generate simulation data
#' dat <- OHPL.sim(
#'   n = 100, p = 100, rho = 0.8,
#'   coef = rep(1, 10), snr = 3, p.train = 0.5,
#'   seed = 1010
#' )
#'
#' # Split training and test set
#' x <- dat$x.tr
#' y <- dat$y.tr
#' x.test <- dat$x.te
#' y.test <- dat$y.te
#'
#' # Fit the OHPL model
#' fit <- OHPL(x, y, maxcomp = 3, gamma = 0.5, G = 10, type = "max")
#'
#' # Compute evaluation metric RMSEP, Q2 and MAE for the test set
#' perf <- OHPL.RMSEP(fit, x.test, y.test)
#' perf$RMSEP
#' perf$Q2
#' perf$MAE
OHPL.RMSEP <- function(object, newx, newy) {
  # Make predictions based on the fitted OHPL model
  y.pred <- predict(
    object,
    newx = newx,
    ncomp = object$"opt.K", type = "response"
  )
  y.pred <- as.matrix(y.pred, ncol = 1L)

  # Compute RMSEP and Q2.test
  newy <- as.matrix(newy, ncol = 1L)
  residual <- y.pred - newy
  RMSEP <- sqrt(mean((residual)^2, na.rm = TRUE))
  MAE <- mean(abs(residual), na.rm = TRUE)
  Q2.test <- 1L - (sum((residual)^2, na.rm = TRUE) / sum((newy - mean(newy))^2))

  res <- list(
    "RMSEP" = RMSEP,
    "MAE" = MAE,
    "Q2.test" = Q2.test,
    "y.pred" = y.pred,
    "residual" = residual
  )

  res
}

#' Extract the prototype from each variable group
#'
#' Extracts the prototype from each variable group.
#'
#' @param X Predictor matrix.
#' @param y Response matrix with one column.
#' @param groups An group index vector containing the
#'   group number each variable belongs to. For example:
#'   `c(1, 1, 1, 1, 1, 2, 2, 2, ...)`.
#'   Variable groups can be generated by the Fisher optimal
#'   partition algorithm implemented in [FOP()].
#' @param type The rule for extracting the prototype.
#'   Possible options are `"max"` and `"median"`.
#' @param mu The mean value of `y` for standardization.
#'   Default is `NULL`, which uses the sample mean of `y`.
#'
#' @return The prototypes (variable index) extracted
#'   from each group (cluster).
#'
#' @export proto
proto <- function(X, y, groups, type = c("max", "median"), mu = NULL) {
  # Check inputs
  if (length(y) != nrow(X)) {
    stop("X and y should have the same number of rows")
  }

  # Extract some parameters # dimensions
  n <- length(y)
  p <- ncol(X)

  # Sort out the data

  # Standardize X
  X.tilde <- apply(X, 2L, function(col) col - mean(col))
  X.tilde <- apply(X.tilde, 2L, function(col) col / sqrt(sum(col^2)))

  # Is mu specified? adjust y accordingly
  y.tilde <- if (is.null(mu)) y - mean(y) else y - mu

  # groups: this is a vector of index which denote the groups
  unique.groups <- unique(groups)
  K <- length(unique.groups)

  # proto.matrix = matrix(NA, n, K)
  col.ind <- NULL

  for (i in 1L:K) {
    var.index <- which(groups == i)
    abs.cor <- abs(t(X.tilde[, var.index]) %*% y.tilde)

    # Find the maximum absolute correlation
    if (type == "max") {
      index <- which.max(abs.cor)
      index <- var.index[index[1]]
      col.ind[i] <- index
      # proto.matrix[, i] = X[, index,drop = FALSE]
    }

    # Find the median of absolute correlation
    if (type == "median") {
      if (length(abs.cor) %% 2 == 1) {
        med.cor <- median(abs.cor)
        index <- which(abs.cor == med.cor)
        med.index <- var.index[index]
        col.ind[i] <- med.index
        # proto.matrix[, i] = x[, med.index, drop = FALSE]
      } else {
        min.index <- which.min(abs.cor)
        abs.cor <- abs.cor[-min.index]
        med.cor <- median(abs.cor)
        index <- which(abs.cor == med.cor)
        med.index <- var.index[index]
        col.ind[i] <- med.index
        # proto.matrix[, i] = x[, med.index, drop = FALSE]
      }
    }
  }

  col.ind
}

dia <- function(X, i, n) {
  sapply(
    (i + 1):n,
    function(x) {
      sum(apply(
        as.matrix(X[i:x, ]), 2, function(y) (sd(y))^2 * (x - i)
      ))
    }
  )
}

# dia <- function(X, i, j) { # correct only when data dimension is 1
#   X <- X[i:j, ]
#   X <- as.matrix(X)
#   mu <- colMeans(X)
#   d <- apply(X, 1, function(x) crossprod(x - mu))
#   sum(d)
# }

#' Ordered Homogeneity Pursuit Lasso
#'
#' Fits the ordered homogeneity pursuit lasso (OHPL) model.
#'
#' @param x Predictor matrix.
#' @param y Response matrix with one column.
#' @param maxcomp Maximum number of components for PLS.
#' @param gamma A number between (0, 1) for generating the gamma sequence.
#'   An usual choice for gamma could be `n * 0.05`, where `n` is a number
#'   in 2, 3, ..., 19.
#' @param cv.folds Number of cross-validation folds.
#' @param G Maximum number of variable groups.
#' @param type Find the maximum absolute correlation (`"max"`)
#'   or find the median of absolute correlation (`"median"`).
#'   Default is `"max"`.
#' @param scale Should the predictor matrix be scaled? Default is `TRUE`.
#' @param pls.method Method for fitting the PLS model. Default is `"simpls"`.
#'   See the details section in [pls::plsr()] for all possible options.
#'
#' @return A list of fitted OHPL model object with performance metrics.
#'
#' @importFrom glmnet cv.glmnet
#'
#' @export OHPL
#'
#' @references
#' You-Wu Lin, Nan Xiao, Li-Li Wang, Chuan-Quan Li, and Qing-Song Xu (2017).
#' Ordered homogeneity pursuit lasso for group variable selection with
#' applications to spectroscopic data.
#' _Chemometrics and Intelligent Laboratory Systems_ 168, 62--71.
#'
#' @examples
#' # Generate simulation data
#' dat <- OHPL.sim(
#'   n = 100, p = 100, rho = 0.8,
#'   coef = rep(1, 10), snr = 3, p.train = 0.5,
#'   seed = 1010
#' )
#'
#' # Split training and test set
#' x <- dat$x.tr
#' y <- dat$y.tr
#' x.test <- dat$x.te
#' y.test <- dat$y.te
#'
#' # Fit the OHPL model
#' fit <- OHPL(x, y, maxcomp = 3, gamma = 0.5, G = 10, type = "max")
#'
#' # Selected variables
#' fit$Vsel
#'
#' # Make predictions
#' y.pred <- predict(fit, x.test)
#'
#' # Compute evaluation metric RMSEP, Q2 and MAE for the test set
#' perf <- OHPL.RMSEP(fit, x.test, y.test)
#' perf$RMSEP
#' perf$Q2
#' perf$MAE
OHPL <- function(
    x, y, maxcomp, gamma, cv.folds = 5L,
    G = 30L, type = c("max", "median"),
    scale = TRUE, pls.method = "simpls") {
  type <- match.arg(type)

  X <- x
  y <- y
  n.cal <- length(y)

  # Compute beta
  beta <- single.beta(X = x, y = y, maxcomp = maxcomp)$"beta"

  # Cluster variables with Fisher optimal partitions algorithm
  C <- dlc(beta, maxk = G)$"C"
  groups <- FOP(beta, G, C)

  # Extract the prototypes from each group
  prototype <- proto(x, y, groups, type = type)

  # X here should be the original X, not the normalized X
  X.pro <- x[, prototype]

  # Lasso with glmnet (with default lambdas)
  lasso.cv <- suppressWarnings(cv.glmnet(X.pro, y, nfolds = cv.folds))

  # optnum: select the model with minimal cv error
  lasso.optnum <- which.min(lasso.cv$cvm)[1]
  lasso.cv.min <- sqrt(min(lasso.cv$cvm)) # cv.min
  lasso.optlambda <- lasso.cv$lambda.min # optlambda
  lasso.optbeta <- lasso.cv$glmnet.fit$beta[, lasso.optnum]
  max.beta <- max(abs(lasso.optbeta))

  # Edge case: when there are less groups, lasso tends to
  # generate many near-zero coefficients (false positives)
  # filter out variables here with hard thresholding.
  index.beta <- (abs(lasso.optbeta) / max.beta) < gamma
  lasso.optbeta[index.beta] <- 0

  # Build PLS model with the selected prototype
  # and the variable grouping information
  index.nozero <- which(lasso.optbeta != 0)
  Vsel <- NULL

  for (i in 1L:length(index.nozero)) {
    Vseli <- which(groups == index.nozero[i])
    Vsel <- c(Vsel, Vseli)
  }

  # Vsel.idx[[j]] = Vsel
  # use the selected variables to build the PLS model
  # note that the X and y here are the original ones (not centered)
  Xsel <- x[, Vsel]

  if (dim(Xsel)[2L] == 0L) {
    Vsel <- c(Vsel, prototype)
    Xsel <- X.pro
    print("The length of selected variables is zero")
  }

  temp.ncomp <- min((dim(Xsel)[1] - 1), dim(Xsel)[2])
  # Xsel.idx[[j]] = Xsel.temp
  # create a new data frame based on the selected
  # varaibles (Xsel) and the original response (y)
  plsdf <- as.data.frame(cbind(Xsel, y = y))

  Vsel.maxcomp <- min(length(Vsel), maxcomp, temp.ncomp)

  plsr.cvfit <- plsr(
    y ~ .,
    data = plsdf, ncomp = Vsel.maxcomp, scale = scale,
    method = pls.method, validation = "LOO"
  )

  # Select the best component number using adjusted CV
  opt.K <- which.min(pls::RMSEP(plsr.cvfit)[["val"]][2L, 1L, -1L])

  # Store the minimal RMSE from CV
  RMSECV <- min(pls::RMSEP(plsr.cvfit)[["val"]][2L, 1L, -1L])

  # Store Q2.CV
  Q2.cv <- R2(plsr.cvfit, estimate = "CV")[["val"]][, , opt.K + 1]

  nVar <- length(Vsel)
  variables <- matrix(0, nrow = ncol(X), ncol = 1)
  variables[Vsel, ] <- 1

  # Build an optimal PLS model
  plsdf <- as.data.frame(cbind(Xsel, y = y))

  plsr.fit <- plsr(
    y ~ Xsel,
    data = plsdf, ncomp = Vsel.maxcomp, scale = scale,
    method = pls.method, validation = "none"
  )

  RMSEC <- min(pls::RMSEP(plsr.fit, estimate = "train")[["val"]][, , opt.K + 1])
  Q2.c <- pls::R2(plsr.fit, estimate = "train")[["val"]][, , opt.K + 1]

  res <- list(
    "model" = plsr.fit, "opt.K" = opt.K,
    "RMSECV" = RMSECV, "Q2.cv" = Q2.cv,
    "RMSEC" = RMSEC, "Q2.c" = Q2.c,
    "nVar" = nVar, "variables" = variables, "Vsel" = Vsel,
    "groups" = groups
  )
  class(res) <- c("OHPL")

  res
}

#' Cross-validation for Ordered Homogeneity Pursuit Lasso
#'
#' Use cross-validation to help select the optimal number
#' of variable groups and the value of `gamma`.
#'
#' @param X.cal Predictor matrix (training).
#' @param y.cal Response matrix with one column (training).
#' @param maxcomp Maximum number of components for PLS.
#' @param gamma A vector of the gamma sequence between (0, 1).
#' @param X.test X.test Predictor matrix (test).
#' @param y.test y.test Response matrix with one column (test).
#' @param cv.folds Number of cross-validation folds.
#' @param G Maximum number of variable groups.
#' @param type Find the maximum absolute correlation (`"max"`)
#'   or find the median of absolute correlation (`"median"`).
#'   Default is `"max"`.
#' @param scale Should the predictor matrix be scaled?
#'   Default is `TRUE`.
#' @param pls.method Method for fitting the PLS model.
#'   Default is `"simpls"`. See the details section
#'   in [pls::plsr()] for all possible options.
#'
#' @return A list containing the optimal model, RMSEP, Q2,
#'   and other evaluation metrics. Also the optimal number
#'   of groups to use in group lasso.
#'
#' @export cv.OHPL
#'
#' @examples
#' data("wheat")
#'
#' X <- wheat$x
#' y <- wheat$protein
#' n <- nrow(wheat$x)
#'
#' set.seed(1001)
#' samp.idx <- sample(1L:n, round(n * 0.7))
#' X.cal <- X[samp.idx, ]
#' y.cal <- y[samp.idx]
#' X.test <- X[-samp.idx, ]
#' y.test <- y[-samp.idx]
#'
#' # This could run for a while
#' \dontrun{
#' cv.fit <- cv.OHPL(
#'   x, y,
#'   maxcomp = 6, gamma = seq(0.1, 0.9, 0.1),
#'   x.test, y.test, cv.folds = 5, G = 30, type = "max"
#' )
#' # the optimal G and gamma
#' cv.fit$opt.G
#' cv.fit$opt.gamma
#' }
cv.OHPL <- function(
    X.cal, y.cal, maxcomp, gamma = seq(0.1, 0.9, 0.1),
    X.test, y.test, cv.folds = 5L,
    G = 30L, type = c("max", "median"),
    scale = TRUE, pls.method = "simpls") {
  type <- match.arg(type)

  RMSEP.mat <- matrix(NA, length(3:G), length(gamma))
  Q2.mat <- matrix(NA, length(3:G), length(gamma))
  # alpha.idx = vector("list", length(gamma))
  # alpha.RMSEP = NULL
  # gamma.idx = vector("list", length(gamma))
  # K.idx = vector("list", length(3:K))
  # K.RMSEP = NULL
  L <- list()

  # Select the optimal gamma and the optimal number of groups
  for (i in 3L:G) {
    for (j in 1L:length(gamma)) {
      ohpl.model <- OHPL(
        X.cal, y.cal, maxcomp,
        gamma = gamma[j],
        cv.folds = cv.folds,
        G = i, type = type,
        scale = scale, pls.method = pls.method
      )

      ohpl.rmsep.res <- OHPL.RMSEP(ohpl.model, X.test, y.test)

      RMSEP.mat[i - 2, j] <- ohpl.rmsep.res$"RMSEP"
      Q2.mat[i - 2, j] <- ohpl.rmsep.res$"Q2.test"

      # Each component in the list stores a model
      L[[(i - 3) * length(gamma) + j]] <- ohpl.model
      cat("gamma =", gamma[j], "\n")
      # alpha.idx[[j]] = ohpl.model
      # alpha.RMSEP = c(alpha.RMSEP, ohpl.model$RMSEP)
    }
    cat("# of clusters =", i, "\n")
    # alpha.RMSEP.ind = which.min(alpha.RMSEP)
    # K.idx[[i-2]] = alpha.idx[[alpha.RMSEP.ind]]
    # K.RMSEP = c(K.RMSEP, alpha.RMSEP[alpha.RMSEP.ind])
  }

  minn <- min(RMSEP.mat)
  # For each column in X, index of the rows which equal to mmin
  a <- apply(RMSEP.mat, 2, function(x) which(x == minn))
  # Get the column where the minimum value is
  b <- sapply(a, function(x) length(x))
  # For each column in X, the number of rows which equals to minn
  # largest value in the index, which has the largest gamma, the sparser model
  gamma.num <- max(which(b > 0L))
  # Select the row with the largest value that corresponds to
  # the column with the largest value
  # to have fewer variables in each block, more concentrated.
  opt.G <- max(a[[gamma.num]]) # optimal number of groups
  opt.gamma <- gamma[gamma.num]
  Q2 <- max(Q2.mat)

  # Select the optimal model
  # Store one model in each component in the list
  result <- L[[(opt.G - 1) * length(gamma) + gamma.num]]

  res <- list("model" = result, "opt.G" = opt.G, "opt.gamma" = opt.gamma)
  res
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OHPL.RMSEP.R
\name{OHPL.RMSEP}
\alias{OHPL.RMSEP}
\title{Compute RMSEP, MAE, and Q2 for a test set}
\usage{
OHPL.RMSEP(object, newx, newy)
}
\arguments{
\item{object}{Object of class \code{OHPL} fitted by \code{\link[=OHPL]{OHPL()}}.}

\item{newx}{Predictor matrix of the new data.}

\item{newy}{Response matrix of the new data (matrix with one column).}
}
\value{
A list of the performance metrics.
}
\description{
Makes predictions on new data and computes the performance evaluation
metrics RMSEP, MAE, and Q2.
}
\examples{
# Generate simulation data
dat <- OHPL.sim(
  n = 100, p = 100, rho = 0.8,
  coef = rep(1, 10), snr = 3, p.train = 0.5,
  seed = 1010
)

# Split training and test set
x <- dat$x.tr
y <- dat$y.tr
x.test <- dat$x.te
y.test <- dat$y.te

# Fit the OHPL model
fit <- OHPL(x, y, maxcomp = 3, gamma = 0.5, G = 10, type = "max")

# Compute evaluation metric RMSEP, Q2 and MAE for the test set
perf <- OHPL.RMSEP(fit, x.test, y.test)
perf$RMSEP
perf$Q2
perf$MAE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dlc.R
\name{dlc}
\alias{dlc}
\title{Compute D, L, and C in the Fisher optimal partitions algorithm}
\usage{
dlc(X, maxk)
}
\arguments{
\item{X}{A set of samples.}

\item{maxk}{Maximum number of \code{k}.}
}
\value{
The D, L, and C statistics in the
Fisher optimal partitions algorithm.
}
\description{
Compute D, L, and C in the Fisher optimal partitions algorithm
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OHPL-datalist.R
\docType{data}
\name{soil}
\alias{soil}
\title{The soil dataset}
\usage{
data(soil)
}
\description{
The soil dataset contains 108 sample measurements from the
wavelength range of 400–2500 nm (visible and near infrared spectrum)
published by Rinnan et al.
}
\examples{
data("soil")

X <- soil$x
y <- soil$som
n <- nrow(soil$x)

set.seed(1001)
samp.idx <- sample(1L:n, round(n * 0.7))
X.cal <- X[samp.idx, ]
y.cal <- y[samp.idx]
X.test <- X[-samp.idx, ]
y.test <- y[-samp.idx]
}
\references{
Rinnan, R., & Rinnan, A. (2007). Application of near infrared
reflectance (NIR) and fluorescence spectroscopy to analysis of
microbiological and chemical properties of arctic soil.
\emph{Soil biology and Biochemistry}, 39(7), 1664--1673.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proto.R
\name{proto}
\alias{proto}
\title{Extract the prototype from each variable group}
\usage{
proto(X, y, groups, type = c("max", "median"), mu = NULL)
}
\arguments{
\item{X}{Predictor matrix.}

\item{y}{Response matrix with one column.}

\item{groups}{An group index vector containing the
group number each variable belongs to. For example:
\code{c(1, 1, 1, 1, 1, 2, 2, 2, ...)}.
Variable groups can be generated by the Fisher optimal
partition algorithm implemented in \code{\link[=FOP]{FOP()}}.}

\item{type}{The rule for extracting the prototype.
Possible options are \code{"max"} and \code{"median"}.}

\item{mu}{The mean value of \code{y} for standardization.
Default is \code{NULL}, which uses the sample mean of \code{y}.}
}
\value{
The prototypes (variable index) extracted
from each group (cluster).
}
\description{
Extracts the prototype from each variable group.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OHPL.R
\name{OHPL}
\alias{OHPL}
\title{Ordered Homogeneity Pursuit Lasso}
\usage{
OHPL(
  x,
  y,
  maxcomp,
  gamma,
  cv.folds = 5L,
  G = 30L,
  type = c("max", "median"),
  scale = TRUE,
  pls.method = "simpls"
)
}
\arguments{
\item{x}{Predictor matrix.}

\item{y}{Response matrix with one column.}

\item{maxcomp}{Maximum number of components for PLS.}

\item{gamma}{A number between (0, 1) for generating the gamma sequence.
An usual choice for gamma could be \code{n * 0.05}, where \code{n} is a number
in 2, 3, ..., 19.}

\item{cv.folds}{Number of cross-validation folds.}

\item{G}{Maximum number of variable groups.}

\item{type}{Find the maximum absolute correlation (\code{"max"})
or find the median of absolute correlation (\code{"median"}).
Default is \code{"max"}.}

\item{scale}{Should the predictor matrix be scaled? Default is \code{TRUE}.}

\item{pls.method}{Method for fitting the PLS model. Default is \code{"simpls"}.
See the details section in \code{\link[pls:mvr]{pls::plsr()}} for all possible options.}
}
\value{
A list of fitted OHPL model object with performance metrics.
}
\description{
Fits the ordered homogeneity pursuit lasso (OHPL) model.
}
\examples{
# Generate simulation data
dat <- OHPL.sim(
  n = 100, p = 100, rho = 0.8,
  coef = rep(1, 10), snr = 3, p.train = 0.5,
  seed = 1010
)

# Split training and test set
x <- dat$x.tr
y <- dat$y.tr
x.test <- dat$x.te
y.test <- dat$y.te

# Fit the OHPL model
fit <- OHPL(x, y, maxcomp = 3, gamma = 0.5, G = 10, type = "max")

# Selected variables
fit$Vsel

# Make predictions
y.pred <- predict(fit, x.test)

# Compute evaluation metric RMSEP, Q2 and MAE for the test set
perf <- OHPL.RMSEP(fit, x.test, y.test)
perf$RMSEP
perf$Q2
perf$MAE
}
\references{
You-Wu Lin, Nan Xiao, Li-Li Wang, Chuan-Quan Li, and Qing-Song Xu (2017).
Ordered homogeneity pursuit lasso for group variable selection with
applications to spectroscopic data.
\emph{Chemometrics and Intelligent Laboratory Systems} 168, 62--71.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OHPL-datalist.R
\docType{data}
\name{beer}
\alias{beer}
\title{The beer dataset}
\usage{
data(beer)
}
\description{
The beer dataset contains 60 samples published by Norgaard et al.
Recorded with a 30mm quartz cell on the undiluted degassed beer
and measured from 1100 to 2250 nm (576 data points) in steps of 2 nm.
}
\examples{
data("beer")
x.cal <- beer$xtrain
dim(x.cal)
x.test <- beer$xtest
dim(x.test)
y.cal <- beer$ytrain
dim(y.cal)
y.test <- beer$ytest
dim(y.test)

X <- rbind(x.cal, x.test)
y <- rbind(y.cal, y.test)
n <- nrow(y)

set.seed(1001)
samp.idx <- sample(1L:n, round(n * 0.7))
X.cal <- X[samp.idx, ]
y.cal <- y[samp.idx]
X.test <- X[-samp.idx, ]
y.test <- y[-samp.idx]
}
\references{
Norgaard, L., Saudland, A., Wagner, J., Nielsen, J. P., Munck, L., &
Engelsen, S. B. (2000). Interval partial least-squares regression (iPLS):
a comparative chemometric study with an example from near-infrared
spectroscopy. \emph{Applied Spectroscopy}, 54(3), 413--419.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.OHPL.R
\name{predict.OHPL}
\alias{predict.OHPL}
\title{Make predictions based on the fitted OHPL model}
\usage{
\method{predict}{OHPL}(object, newx, ncomp = NULL, type = "response", ...)
}
\arguments{
\item{object}{Object of class \code{OHPL} fitted by \code{\link[=OHPL]{OHPL()}}.}

\item{newx}{Predictor matrix of the new data.}

\item{ncomp}{Optimal number of components.
If is \code{NULL}, the optimal number of components
stored in the model object will be used.}

\item{type}{Prediction type.}

\item{...}{Additional parameters.}
}
\value{
Numeric matrix of the predicted values.
}
\description{
Make predictions on new data by an OHPL model object.
}
\examples{
# generate simulation data
dat <- OHPL.sim(
  n = 100, p = 100, rho = 0.8,
  coef = rep(1, 10), snr = 3, p.train = 0.5,
  seed = 1010
)

# split training and test set
x <- dat$x.tr
y <- dat$y.tr
x.test <- dat$x.te
y.test <- dat$y.te

# fit the OHPL model
fit <- OHPL(x, y, maxcomp = 3, gamma = 0.5, G = 10, type = "max")
# make predictions
y.pred <- predict(fit, x.test)
y.pred
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OHPL.sim.R
\name{OHPL.sim}
\alias{OHPL.sim}
\title{Generate simulation data for benchmarking sparse regressions
(Gaussian response)}
\usage{
OHPL.sim(
  n = 100,
  p = 100,
  rho = 0.8,
  coef = rep(1, 10),
  snr = 3,
  p.train = 0.5,
  seed = 1001
)
}
\arguments{
\item{n}{Number of observations.}

\item{p}{Number of variables.}

\item{rho}{Correlation base for generating correlated variables.}

\item{coef}{Vector of non-zero coefficients.}

\item{snr}{Signal-to-noise ratio (SNR).}

\item{p.train}{Percentage of training set.}

\item{seed}{Random seed for reproducibility.}
}
\value{
A list containing \code{x.tr}, \code{x.te}, \code{y.tr}, and \code{y.te}.
}
\description{
Generate simulation data (Gaussian case) following the
settings in Xiao and Xu (2015).
}
\examples{
dat <- OHPL.sim(
  n = 100, p = 100, rho = 0.8,
  coef = rep(1, 10), snr = 3, p.train = 0.5,
  seed = 1010
)

dim(dat$x.tr)
dim(dat$x.te)
}
\references{
Nan Xiao and Qing-Song Xu. (2015). Multi-step adaptive elastic-net:
reducing false positives in high-dimensional variable selection.
\emph{Journal of Statistical Computation and Simulation} 85(18), 3755--3765.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OHPL-package.R
\docType{package}
\name{OHPL-package}
\alias{OHPL-package}
\title{OHPL: Ordered Homogeneity Pursuit Lasso for Group Variable Selection}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Ordered homogeneity pursuit lasso (OHPL) algorithm for group variable selection proposed in Lin et al. (2017) \doi{10.1016/j.chemolab.2017.07.004}. The OHPL method exploits the homogeneity structure in high-dimensional data and enjoys the grouping effect to select groups of important variables automatically. This feature makes it particularly useful for high-dimensional datasets with strongly correlated variables, such as spectroscopic data.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ohpl.io}
  \item \url{https://ohpl.io/doc/}
  \item \url{https://github.com/nanxstats/OHPL}
  \item Report bugs at \url{https://github.com/nanxstats/OHPL/issues}
}

}
\author{
\strong{Maintainer}: Nan Xiao \email{me@nanx.me} (\href{https://orcid.org/0000-0002-0250-5673}{ORCID})

Authors:
\itemize{
  \item You-Wu Lin \email{lyw015813@126.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OHPL-datalist.R
\docType{data}
\name{wheat}
\alias{wheat}
\title{The wheat dataset}
\usage{
data(wheat)
}
\description{
The wheat dataset contains 100 wheat samples with specified
protein and moisture content, published by J. Kalivas.
Samples were measured by diffuse reflectance as log (I/R)
from 1100 to 2500 nm (701 data points) in 2 nm intervals.
}
\examples{
data("wheat")

X <- wheat$x
y <- wheat$protein
n <- nrow(wheat$x)

set.seed(1001)
samp.idx <- sample(1L:n, round(n * 0.7))
X.cal <- X[samp.idx, ]
y.cal <- y[samp.idx]
X.test <- X[-samp.idx, ]
y.test <- y[-samp.idx]
}
\references{
Kalivas, J. H. (1997). Two data sets of near infrared spectra.
\emph{Chemometrics and Intelligent Laboratory Systems}, 37(2), 255--259.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FOP.R
\name{FOP}
\alias{FOP}
\title{Fisher optimal partition}
\usage{
FOP(X, k, C)
}
\arguments{
\item{X}{A set of samples.}

\item{k}{Number of classes.}

\item{C}{Statistic from the output of \code{\link[=dlc]{dlc()}}.}
}
\value{
Index vector for each sample's classification.
}
\description{
The Fisher optimal partition algorithm.
}
\examples{
X <- matrix(c(
  9.3, 1.8, 1.9, 1.7, 1.5, 1.3,
  1.4, 2.0, 1.9, 2.3, 2.1
))
C <- dlc(X, maxk = 8)$C
F <- FOP(X, 8, C)
}
\references{
W. D. Fisher (1958). On grouping for maximum homogeneity.
\emph{Journal of the American Statistical Association},
vol. 53, pp. 789--798.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv.OHPL.R
\name{cv.OHPL}
\alias{cv.OHPL}
\title{Cross-validation for Ordered Homogeneity Pursuit Lasso}
\usage{
cv.OHPL(
  X.cal,
  y.cal,
  maxcomp,
  gamma = seq(0.1, 0.9, 0.1),
  X.test,
  y.test,
  cv.folds = 5L,
  G = 30L,
  type = c("max", "median"),
  scale = TRUE,
  pls.method = "simpls"
)
}
\arguments{
\item{X.cal}{Predictor matrix (training).}

\item{y.cal}{Response matrix with one column (training).}

\item{maxcomp}{Maximum number of components for PLS.}

\item{gamma}{A vector of the gamma sequence between (0, 1).}

\item{X.test}{X.test Predictor matrix (test).}

\item{y.test}{y.test Response matrix with one column (test).}

\item{cv.folds}{Number of cross-validation folds.}

\item{G}{Maximum number of variable groups.}

\item{type}{Find the maximum absolute correlation (\code{"max"})
or find the median of absolute correlation (\code{"median"}).
Default is \code{"max"}.}

\item{scale}{Should the predictor matrix be scaled?
Default is \code{TRUE}.}

\item{pls.method}{Method for fitting the PLS model.
Default is \code{"simpls"}. See the details section
in \code{\link[pls:mvr]{pls::plsr()}} for all possible options.}
}
\value{
A list containing the optimal model, RMSEP, Q2,
and other evaluation metrics. Also the optimal number
of groups to use in group lasso.
}
\description{
Use cross-validation to help select the optimal number
of variable groups and the value of \code{gamma}.
}
\examples{
data("wheat")

X <- wheat$x
y <- wheat$protein
n <- nrow(wheat$x)

set.seed(1001)
samp.idx <- sample(1L:n, round(n * 0.7))
X.cal <- X[samp.idx, ]
y.cal <- y[samp.idx]
X.test <- X[-samp.idx, ]
y.test <- y[-samp.idx]

# This could run for a while
\dontrun{
cv.fit <- cv.OHPL(
  x, y,
  maxcomp = 6, gamma = seq(0.1, 0.9, 0.1),
  x.test, y.test, cv.folds = 5, G = 30, type = "max"
)
# the optimal G and gamma
cv.fit$opt.G
cv.fit$opt.gamma
}
}
